import 'package:smart_arrays_numerics/smart_arrays_numerics.dart';

void main() {
  Numerics nu = Numerics();

  int testNumber = 1;
  int failed = 0;

  /// Call this for each test
  void test(String testName, Function testFunction) {
    print("${testNumber++}: $testName");
    testFunction();
  }

  /// Test data
  final List<List<double>> MAT0 = [
    [1, 1, 1, 0],
    [0, 3, 1, 2],
    [2, 3, 1, 0],
    [1, 0, 2, 1]
  ],
      INVERSE0 = [
    [-3, -0.5, 1.5, 1],
    [1, 0.25, -0.25, -0.5],
    [3, 0.25, -1.25, -0.5],
    [-3, -0.0, 1.0, 1.0]
  ],
      UNITY0 = [
    [1, 0, 0, 0],
    [0, 1, 0, 0],
    [0, 0, 1, 0],
    [0, 0, 0, 1]
  ],
      MAT1 = [
    [1, 1, 1],
    [0, 3, 1],
    [2, 3, 1],
    [1, 0, 2]
  ],
      TRANSPOSED1 = [
    [1, 0, 2, 1],
    [1, 3, 3, 0],
    [1, 1, 1, 2]
  ];

  /// Print a matrix-matrix test result: [comparison] has the for
  /// "row, col, computed value, expected value" in case of an error,
  /// otherwise null.
  printResultMM(String comparison) {
    if (comparison != null) {
      print(
          "Failed: row, col, computed element, expected element: $comparison");
      failed++;
      return;
    }
    print("Passed.");
  }

  test("Numerics.invMatrix:", () {
    List<List<double>> computedInverse = nu.invMatrix(MAT0);
    String comp = nu.compareMM(computedInverse, INVERSE0, 1e-8);
    printResultMM(comp);
  });

  test("Numerics.transpose:", () {
    List<List<double>> computedTransposed = nu.transpose(MAT1);
    String comp = nu.compareMM(computedTransposed, TRANSPOSED1, 1e-8);
    printResultMM(comp);
  });

  test("Numerics.dotMM:", () {
    List<List<double>> computedMM = nu.dotMM(MAT0, INVERSE0);
    String comp = nu.compareMM(computedMM, UNITY0, 1e-8);
    printResultMM(comp);
  });

  if (failed == 0) {
    print("All tests passed");
  } else {
    print("$failed tests failed");
  }
}
